# coding=utf-8
from unittest import TestCase

from earthdiagnostics.diagnostic import DiagnosticVariableOption
from earthdiagnostics.box import Box
from earthdiagnostics.general.relinkall import RelinkAll
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms


class TestRelinkAll(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))
        self.diags.config.experiment.startdates = ['20010101', ]

        self.box = Box()
        self.box.min_depth = 0
        self.box.max_depth = 100

    def fake_parse(self, value):
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = RelinkAll.generate_jobs(self.diags, ['diagnostic'])
        self.assertEqual(len(jobs), 1)
        self.assertEqual(jobs[0], RelinkAll(self.data_manager, '20010101'))

        with self.assertRaises(Exception):
            RelinkAll.generate_jobs(self.diags, ['diagnostic', '0'])

    def test_str(self):
        mixed = RelinkAll(self.data_manager, '20010101')
        self.assertEquals(str(mixed), 'Relink all output Startdate: 20010101')
