# coding=utf-8
from unittest import TestCase

from earthdiagnostics.box import Box
from earthdiagnostics.ocean.averagesection import AverageSection
from mock import Mock, patch
from earthdiagnostics.diagnostic import DiagnosticVariableOption

from earthdiagnostics.modelingrealm import ModelingRealms


class TestAverageSection(TestCase):

    def setUp(self):
        self.data_manager = Mock()
        self.diags = Mock()

        self.box = Box()
        self.box.min_lat = 0
        self.box.max_lat = 0
        self.box.min_lon = 0
        self.box.max_lon = 0

        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))

    def fake_parse(self, value):
        return value

    # noinspection PyUnresolvedReferences
    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = AverageSection.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', '0', '0', '0', '0'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], AverageSection(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                                 self.box, ''))
        self.assertEqual(jobs[1], AverageSection(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                                 self.box, ''))

        jobs = AverageSection.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', '0', '0', '0', '0', 'grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], AverageSection(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                                 self.box, 'grid'))
        self.assertEqual(jobs[1], AverageSection(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                                 self.box, 'grid'))

        with self.assertRaises(Exception):
            AverageSection.generate_jobs(self.diags, ['diagnostic'])
        with self.assertRaises(Exception):
            AverageSection.generate_jobs(self.diags, ['diagnostic', 'ocean', 'var', '0', '0', '0', '0', 'grid',
                                                      'extra'])

    def test_str(self):
        diag = AverageSection(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var', self.box, 'grid')
        self.assertEquals(str(diag), 'Average section Startdate: 20010101 Member: 0 Chunk: 0 Box: 0N0E '
                                     'Variable: ocean:var Grid: grid')
