# coding=utf-8
from unittest import TestCase

from earthdiagnostics.ocean.interpolate import Interpolate
from mock import Mock, patch

from earthdiagnostics.modelingrealm import ModelingRealms
from earthdiagnostics.diagnostic import DiagnosticVariableListOption


class TestInterpolate(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.model_version = 'model_version'
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))
        self.diags.config.experiment.model_version = 'model_version'

    def fake_parse(self, value):
        return value.split('-')

    @patch.object(DiagnosticVariableListOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = Interpolate.generate_jobs(self.diags, ['interp', 'grid', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var', 'grid',
                                              'model_version', False, ''))
        self.assertEqual(jobs[1], Interpolate(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var', 'grid',
                                              'model_version', False, ''))

        jobs = Interpolate.generate_jobs(self.diags, ['interp', 'grid', 'var1-var2'])
        self.assertEqual(len(jobs), 4)
        self.assertEqual(jobs[0], Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var1', 'grid',
                                              'model_version', False, ''))
        self.assertEqual(jobs[1], Interpolate(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var1', 'grid',
                                              'model_version', False, ''))
        self.assertEqual(jobs[2], Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var2', 'grid',
                                              'model_version', False, ''))
        self.assertEqual(jobs[3], Interpolate(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var2', 'grid',
                                              'model_version', False, ''))

        jobs = Interpolate.generate_jobs(self.diags, ['interp', 'grid', 'var', 'atmos'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'grid',
                                              'model_version', False, ''))
        self.assertEqual(jobs[1], Interpolate(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var', 'grid',
                                              'model_version', False, ''))

        jobs = Interpolate.generate_jobs(self.diags, ['interp', 'grid', 'var', 'atmos', 'true'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'grid',
                                              'model_version', True, ''))
        self.assertEqual(jobs[1], Interpolate(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var', 'grid',
                                              'model_version', True, ''))

        jobs = Interpolate.generate_jobs(self.diags, ['interp', 'grid', 'var', 'atmos', 'true', 'original_grid'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'grid',
                                              'model_version', True, 'original_grid'))
        self.assertEqual(jobs[1], Interpolate(self.data_manager, '20010101', 0, 1, ModelingRealms.atmos, 'var', 'grid',
                                              'model_version', True, 'original_grid'))

        with self.assertRaises(Exception):
            Interpolate.generate_jobs(self.diags, ['interp'])

        with self.assertRaises(Exception):
            Interpolate.generate_jobs(self.diags, ['interp', 'grid', 'var', 'atmos', 'true', 'original_grid', 'extra'])

    def test_str(self):
        diag = Interpolate(self.data_manager, '20010101', 0, 0, ModelingRealms.atmos, 'var', 'grid',
                           'model_version', True, 'original_grid')
        self.assertEquals(str(diag), 'Interpolate Startdate: 20010101 Member: 0 Chunk: 0 Variable: atmos:var '
                                     'Target grid: grid Invert lat: True Model: model_version '
                                     'Original grid: original_grid')
