# coding=utf-8
from unittest import TestCase

from earthdiagnostics.box import Box
from earthdiagnostics.constants import Basins, Basin
from earthdiagnostics.ocean.maxmoc import MaxMoc
from mock import Mock, patch


class TestMaxMoc(TestCase):

    def setUp(self):
        self.data_manager = Mock()
        self.basins = Mock()
        self.basins.Global = Basin('Global')
        self.basins.Atlantic = Basin('Atlantic')

        self.box = Box(True)
        self.box.min_lat = 0.0
        self.box.max_lat = 0.0
        self.box.min_depth = 0.0
        self.box.max_depth = 0.0

        self.maxmoc = MaxMoc(self.data_manager, '20000101', 1, 2000, self.basins.Global, self.box)

    def fake_parse(self, value):
        if type(value) is Basin:
            return value
        if value == 'atl':
            value = 'Atlantic'
        else:
            value = 'Global'

        return Basin(value)

    @patch.object(Basins, 'parse', fake_parse)
    def test_generate_jobs(self):
        self.diags = Mock()
        self.diags.model_version = 'model_version'
        self.diags.config.experiment.startdates = ('20010101',)
        self.diags.config.experiment.members = (0,)
        self.diags.config.experiment.get_full_years.return_value = (2000, 2001)

        jobs = MaxMoc.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], MaxMoc(self.data_manager, '20010101', 0, 2000, self.basins.Global, self.box))
        self.assertEqual(jobs[1], MaxMoc(self.data_manager, '20010101', 0, 2001, self.basins.Global, self.box))

        jobs = MaxMoc.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0', 'atl'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], MaxMoc(self.data_manager, '20010101', 0, 2000, self.basins.Atlantic, self.box))
        self.assertEqual(jobs[1], MaxMoc(self.data_manager, '20010101', 0, 2001, self.basins.Atlantic, self.box))

        self.diags.config.experiment.get_full_years.return_value = list()
        jobs = MaxMoc.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0'])
        self.assertEqual(len(jobs), 0)

        with self.assertRaises(Exception):
            MaxMoc.generate_jobs(self.diags, ['diagnostic'])

        with self.assertRaises(Exception):
            MaxMoc.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0', '0', '0', '0'])

    def test_str(self):
        self.assertEquals(str(self.maxmoc), 'Max moc Startdate: 20000101 Member: 1 Year: 2000 '
                                            'Box: 0.0N0.0m Basin: Global')
