# coding=utf-8
from unittest import TestCase

from earthdiagnostics.diagnostic import DiagnosticVariableOption, DiagnosticOptionError
from earthdiagnostics.box import Box
from earthdiagnostics.ocean.verticalmeanmeters import VerticalMeanMeters
from earthdiagnostics.modelingrealm import ModelingRealms
from mock import Mock, patch


class TestVerticalMeanMeters(TestCase):

    def setUp(self):
        self.data_manager = Mock()

        self.diags = Mock()
        self.diags.model_version = 'model_version'
        self.diags.config.experiment.get_chunk_list.return_value = (('20010101', 0, 0), ('20010101', 0, 1))

        self.box = Box(True)
        self.box.min_depth = 0
        self.box.max_depth = 100

        self.mixed = VerticalMeanMeters(self.data_manager, '20000101', 1, 1, ModelingRealms.ocean, 'var', self.box, 'T')

    def fake_parse(self, value):
        if not value:
            raise DiagnosticOptionError
        return value

    @patch.object(DiagnosticVariableOption, 'parse', fake_parse)
    def test_generate_jobs(self):
        jobs = VerticalMeanMeters.generate_jobs(self.diags, ['diagnostic', 'var', '0', '100'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], VerticalMeanMeters(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                                     self.box, 'T'))
        self.assertEqual(jobs[1], VerticalMeanMeters(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                                     self.box, 'T'))

        jobs = VerticalMeanMeters.generate_jobs(self.diags, ['diagnostic', 'var', '0'])
        box = Box(True)
        box.min_depth = 0
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], VerticalMeanMeters(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                                     box, 'T'))
        self.assertEqual(jobs[1], VerticalMeanMeters(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                                     box, 'T'))

        jobs = VerticalMeanMeters.generate_jobs(self.diags, ['diagnostic', 'var'])
        self.assertEqual(len(jobs), 2)
        self.assertEqual(jobs[0], VerticalMeanMeters(self.data_manager, '20010101', 0, 0, ModelingRealms.ocean, 'var',
                                                     Box(True), 'T'))
        self.assertEqual(jobs[1], VerticalMeanMeters(self.data_manager, '20010101', 0, 1, ModelingRealms.ocean, 'var',
                                                     Box(True), 'T'))

        with self.assertRaises(DiagnosticOptionError):
            VerticalMeanMeters.generate_jobs(self.diags, ['diagnostic'])

        with self.assertRaises(DiagnosticOptionError):
            VerticalMeanMeters.generate_jobs(self.diags, ['diagnostic', '0', '0', '0', '0', '0', '0', '0'])

    def test_str(self):
        self.assertEquals(str(self.mixed), 'Vertical mean meters Startdate: 20000101 Member: 1 Chunk: 1 '
                                           'Variable: ocean:var Box: 0-100m')
